@@ -4,6 +4,11 @@ import pandas as pd
 import datetime
 from IPython.display import clear_output
 import plotly.express as px
+import time
+from sqlalchemy import create_engine
+from sqlalchemy.exc import IntegrityError
+import mysql.connector
+from sqlalchemy import create_engine
 
 """# Vista APIs (Raw)  
 -- Lowest Level ( JSON API response, as is )
@@ -123,7 +128,7 @@ vista_search_locations_json()
 ```
 """
 
-def vista_search_locations_json(locations=["lonavala,%20maharashtra"],guests=2,get_all=False):
+def vista_search_locations_json(locations=["lonavala,%20maharashtra"],guests=2,get_all=False,wait_time=10):
 
   # Empty list to append (extend) all the data
   properties = []
@@ -152,12 +157,14 @@ def vista_search_locations_json(locations=["lonavala,%20maharashtra"],guests=2,g
       properties.extend(search_data['data']['properties'])
       page_no += 1
 
+      time.sleep(wait_time)
+
 
   return properties
 
 # Retruns a DATAFRAME for the above functions & **DROP DUPLICATES (always use this for analysis)
-def vista_search_locations(locations=["lonavala,%20maharashtra"],guests=2,get_all=False):
-  villas = vista_search_locations_json(locations=locations, guests=guests,get_all=get_all)
+def vista_search_locations(locations=["lonavala,%20maharashtra"],guests=2,get_all=False,wait_time=10):
+  villas = vista_search_locations_json(locations=locations, guests=guests,get_all=get_all,wait_time=wait_time)
   villas = pd.DataFrame(villas)
   villas = villas.drop_duplicates('id')
 
@@ -191,7 +198,7 @@ def vista_listing(slug='the-boulevard-villa',guests=2,checkin=datetime.date.toda
   # Add the extra fields in response (JSON)
   property_deets['data']['slug'] = slug
   property_deets['data']['lat'] = lat_long['latitude']
-  property_deets['data']['long'] = lat_long['longitude']
+  property_deets['data']['lon'] = lat_long['longitude']
   property_deets['data']['checkin_date'] = checkin
   property_deets['data']['checkout_date'] = checkout
   property_deets['data']['weekday_pricing'] = weekday_pricing
@@ -374,7 +381,6 @@ def vista_states_stacked_bar(mapbox_token='pk.eyJ1Ijoic2FyYW5ncHVyYW5kYXJlIiwiYS
 
 ```
 next_weekday()
-PROXY FOR SEARCH API
 ```
 """
 
@@ -386,81 +392,6 @@ def next_weekday(weekday=0, d=datetime.date.today()):
 
 # default - next monday
 
-from bs4 import BeautifulSoup
-from random import choice
-
-def get_proxy():
-  url = 'https://www.sslproxies.org/'
-  r = requests.get(url)
-  soup = BeautifulSoup(r.content, 'html5lib')
-  return {'https': choice(list(map(lambda x:x[0]+':'+x[1], list(zip(map(lambda x:x.text, soup.findAll('td')[::8]), map(lambda x:x.text, soup.findAll('td')[1::8]))))))}
-
-def proxy_request(request_type, url, **kwargs):
-  while 1:
-    try:
-      proxy = get_proxy()
-      r = requests.request(request_type, url, proxy=proxies, timeout=5, **kwargs)
-      break 
-      return r
-    except:
-      pass
-
-def vista_search_api_proxy(search_type='city',location="lonavala,%20maharashtra",guests=2,adults=2,childs=0,page_no=1):
-
-  url = "https://searchapi.vistarooms.com/api/search/getresults"
-
-  param={
-    }
-
-  proxies = get_proxy()
-
-  proxies = {
-      "https": "103.216.82.52:6666"
-  }
-
-  payload = {
-        
-        "city": location,
-        "search_type": "city",
-        "checkin": "",
-        "checkout": "",
-        "total_guests": guests,
-        "adults": adults,
-        "childs": childs,
-        "page": 1,
-        "min_bedrooms": 1,
-        "max_bedrooms": 30,
-        "amenity": [],
-        "facilities": [],
-        "price_start": 1000,
-        "price_end": 5000000,
-        "sort_by_price": ""
-          
-      }
-
-  payload = "{\n    \"city\": \"lonavala, maharashtra\",\n    \"search_type\": \"city\",\n    \"checkin\": \"\",\n    \"checkout\": \"\",\n    \"total_guests\": 1,\n    \"adults\": 1,\n    \"childs\": 0,\n    \"page\": 1,\n    \"min_bedrooms\": 1,\n    \"max_bedrooms\": 30,\n    \"amenity\": [],\n    \"facilities\": [],\n    \"price_start\": 1000,\n    \"price_end\": 5000000,\n    \"sort_by_price\": \"\"\n}"
-  
-  headers = {
-      'authority': 'searchapi.vistarooms.com',
-      'pragma': 'no-cache',
-      'cache-control': 'no-cache',
-      'accept': 'application/json, text/plain, */*',
-      'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.105 Safari/537.36',
-      'content-type': 'application/json',
-      'origin': 'https://www.vistarooms.com',
-      'sec-fetch-site': 'same-site',
-      'sec-fetch-mode': 'cors',
-      'sec-fetch-dest': 'empty',
-      'referer': 'https://www.vistarooms.com/maharashtra/villas-in-lonavala',
-      'accept-language': 'en-GB,en-US;q=0.9,en;q=0.8',
-      'Cookie': '__cfduid=d6e43e667320a570f3748afb4cee79f6b1596831185; laravel_session=Yvb89jaR2AGpblmNqYaNmuNNbhMay5j8btAzlZr1'
-    }
-
-  response = requests.post(url, params=param, headers=headers, data=payload, proxies=proxies)
-  search_data = response.text.encode('utf8')
-
-  return search_data
-
 """# Get Saved Data ( DataFrames on the cloud )
 Use these to avoid the fetching time (Updated every week)
 """
@@ -495,6 +426,9 @@ Only execute weekly
 
 ```
 vista_get_entire_dataframe()
+vista_weekly_update_script(sql_user,sql_pw,sql_db,sql_db_url,search_api_wait=10)
+^^^ Run this every week without fail
+
 ```
 """
 
@@ -505,6 +439,81 @@ def vista_get_entire_dataframe():
   vista_final_dataframe = vista_master_dataframe(slugs)
   return vista_final_dataframe
 
+def vista_weekly_update_script(sql_user,sql_pw,sql_db,sql_db_url,search_api_wait=10)
+
+    vista_search_data = vista_search_locations(get_all=True,wait_time=search_api_wait)
+
+    # Get the list of all the current villas lited
+    new_slugs = vista_search_data['slug'].values
+
+    #Initialize a db connector
+    mydb = mysql.connector.connect(
+      host=sql_db_url,
+      user=sql_user,
+      password=sql_pw,
+      database=sql_db
+    )
+
+    mycursor = mydb.cursor()
+
+    #Execute the SQL query to fetch the avaiable slugs in our database
+    mycursor.execute("SELECT slug FROM VISTA_MASTER")
+
+    myresult = mycursor.fetchall()
+
+    old_slugs = pd.DataFrame(myresult)
+    old_slugs = old_slugs[0].values
+
+
+    # Get the list of recently added and removed slugs
+    added_slugs = list(set(new_slugs).difference(set(old_slugs)))
+    removed_slugs = list(set(old_slugs).difference(set(new_slugs)))
+
+    #Steps to Add the new listings to the Database
+    vista_newly_added_df = added_villas_dataframe(old_slugs,new_slugs)
+
+
+    if len(vista_newly_added_df) > 0:
+        vista_newly_added_df['listing_status'] = "LISTED"
+        vista_newly_added_df['status_on'] = datetime.datetime.today()
+
+        # changind all the "Object" data types to str (to avoid some weird error in SQL)
+        all_object_types = pd.DataFrame(vista_newly_added_df.dtypes)
+        all_object_types = all_object_types[all_object_types[0]=='object'].index
+
+        for column in all_object_types:
+        vista_newly_added_df[column] = vista_newly_added_df[column].astype('str')
+
+        engine = create_engine(f"mysql+pymysql://{sql_user}:{sql_pw}@{sql_db_url}/{sql_db}")
+
+        for i in range(len(vista_newly_added_df)):
+        try:
+            vista_newly_added_df.iloc[i:i+1].to_sql(name='VISTA_MASTER',if_exists='append',con = engine,index=False)
+        except IntegrityError:
+            pass  
+
+
+    # Update Delisted homes in a different table
+
+    if len(removed_slugs)>0:   
+
+        engine = create_engine(f"mysql+pymysql://{sql_user}:{sql_pw}@{sql_db_url}/{sql_db}")
+
+        sql_query_get_all = "SELECT id,slug,lat,lon,property_type,number_of_rooms,photos,price_per_room,weekday_pricing_value,weekend_pricing_value,entire_week_pricing_value,entire_month_pricing_value,status_on FROM VISTA_MASTER"
+        vista_all = pd.read_sql(sql_query_get_all,engine)
+        vista_delisted = vista_all[vista_all['slug'].isin(removed_slugs)]
+        vista_delisted['delisted_on'] = datetime.datetime.today()
+
+        for i in range(len(vista_delisted)):
+            try:
+                vista_delisted.iloc[i:i+1].to_sql(name='VISTA_DELISTED',if_exists='append',con = engine,index=False)
+            except IntegrityError:
+                pass  
+
+
+    engine.dispose()
+    mycursor.close()
+
 """# TEST"""
 
 def final_test():
@@ -524,4 +533,7 @@ def final_test():
 
   return "All Good :)"
 
-"""# End"""
\ No newline at end of file
+"""# End"""
+
+
+
